
import urllib3
import logging

from promium.base import Element
from promium.helpers import ParseUrl


log = logging.getLogger(__name__)


class Link(Element):

    @property
    def href(self):
        return self.get_attribute("href")

    @property
    def parse_url(self):
        """
        Using parse_url to parse href. Can get attributes:
        - scheme
        - host
        - sub_domain
        - port
        - path
        - params
        - query
        - fragment
        - product_id
        """
        return ParseUrl(self.href)

    @property
    def response(self):
        """Get url response"""
        http = urllib3.PoolManager(
            cert_reqs=False,
            timeout=5,
        )
        cookies_str = '; '.join([
            f'{cookie["name"]}={cookie["value"]}'
            for cookie in self.driver.get_cookies()
        ])
        for i in range(1, 3):
            r = http.request(
                url=self.href,
                method='GET',
                headers={
                    "Accept-Encoding": 'gzip, deflate',
                    "Accept": "*/*",
                    "Connection": "keep-alive",
                    "Cookie": cookies_str,
                },
            )
            if r.status in [502, 503, 504]:
                log.warning(f'[Request Retry]: attempt:{i}, status: {r.status}')
            else:
                break

        return r

    @property
    def get_status_code(self):
        """Gets status response code from link"""
        return self.response.status




import pytest

from promium.device_config import ANDROID_360_640, XIAOMI_REDMI_7
from promium.test_case import WebDriverTestCase
from tests.pages.catalog_gallery_page import CatalogGalleryPage


@pytest.mark.se
class TestCatalogGallery(WebDriverTestCase):
    test_case_url = 'some url with test case'

    def test_view_products_gallery_blocks(self):
        catalog_gallery_page = CatalogGalleryPage(self.driver)
        catalog_gallery_page.open(query='Komplekty-nizhnego-belya')
        catalog_gallery_page.wait_for_page_loaded()
        products_block = catalog_gallery_page.gallery_block
        products_block.wait_to_display(timeout=20)
        products = products_block.product_blocks
        products.first_item.wait_to_display()

    @pytest.mark.device(XIAOMI_REDMI_7)
    @pytest.mark.device(ANDROID_360_640)
    @pytest.mark.parametrize('device', [
        pytest.param('xiaomi', marks=pytest.mark.device(XIAOMI_REDMI_7)),
        pytest.param('android', marks=pytest.mark.device(ANDROID_360_640)),
    ])
    def test_mobile_view_products_gallery_blocks(self, device):
        catalog_gallery_page = CatalogGalleryPage(self.driver)
        catalog_gallery_page.open(query='Komplekty-nizhnego-belya')
        catalog_gallery_page.wait_for_page_loaded()
        products_block = catalog_gallery_page.gallery_block
        products_block.wait_to_display(timeout=20)
        products_block.product_blocks.first_item.wait_to_display()




from selenium import webdriver
from selenium.webdriver import DesiredCapabilities
from selenium.webdriver.common.proxy import Proxy, ProxyType
from selenium.webdriver.remote.remote_connection import RemoteConnection

from selenium.webdriver.chrome.options import Options as ChromeOptions


DOWNLOAD_PATH = "/Downloads"


def get_chrome_opera_options(
        options, device, proxy_server=None, is_headless=False
):
    """
    --whitelisted-ips   Comma-separated whitelist of remote IPv4
                        addresses which are allowed to connect to ChromeDriver,
                        "" - allowed all

    --use-gl            Select which implementation of GL the GPU process
                        should use.

    --no-sandbox        Disables the sandbox for all process types that
                        are normally sandboxed. Meant to be used as
                        a browser-level switch for testing purposes only.
    """
    if proxy_server:
        options.add_argument(f"--proxy-server={proxy_server}")
    if is_headless:
        options.add_argument('--headless=new')
    options.add_argument("--whitelisted-ips=''")
    options.add_argument("--use-gl=swiftshader")  # for rendering Chrome 79+
    options.add_argument("--no-sandbox")
    options.add_argument("--allow-insecure-localhost")
    options.add_argument("--no-default-browser-check")
    options.add_argument("--dns-prefetch-disable")
    options.add_argument("--no-first-run")
    options.add_argument("--verbose")
    options.add_argument("--enable-logging --v=3")
    options.add_argument("--test-type")
    options.add_argument("--ignore-certificate-errors")
    options.add_argument("--disable-notifications")
    options.add_argument("--dns-prefetch-disable")

    if device.__class__.__name__ == "CustomDevice":
        mobile_emulation = {
            "deviceMetrics": {
                "width": device.width,
                "height": device.height,
                "pixelRatio": device.pixel_ratio
            },
            "userAgent": device.user_agent
        }
        options.add_experimental_option(
            "mobileEmulation", mobile_emulation
        )
        # for seeing the emulation window inside browser
        options.add_argument("--window-size=1920,1080")
    else:
        if device.user_agent:
            options.add_argument(f"--user-agent={device.user_agent}")
        options.add_argument(f"--window-size={device.width},{device.height}")
    return options


def get_chrome_options(
        device, proxy_server=None, is_headless=False, default_options=None
):
    options = default_options if default_options else ChromeOptions()
    return get_chrome_opera_options(options, device, proxy_server, is_headless)


def get_opera_options(device, proxy_server=None):
    options = ChromeOptions()
    opera_options = get_chrome_opera_options(options, device, proxy_server)
    opera_options.binary_location = '/usr/bin/opera'
    return opera_options


def get_firefox_profile(device):
    profile = webdriver.FirefoxProfile()
    profile.set_preference("browser.startup.homepage", "about:blank")
    profile.set_preference("startup.homepage_welcome_url", "about:blank")
    profile.set_preference(
        "startup.homepage_welcome_url.additional", "about:blank"
    )
    profile.set_preference("browser.download.folderList", 2)
    profile.set_preference("browser.download.manager.showWhenStarting", False)
    profile.set_preference("browser.download.dir", DOWNLOAD_PATH)
    profile.set_preference(
        "browser.helperApps.neverAsk.saveToDisk",
        "application/zip"
    )
    profile.set_preference("pdfjs.disabled", True)
    profile.set_preference("extensions.firebug.allPagesActivation", "on")
    profile.set_preference("extensions.firebug.console.enableSites", "on")
    profile.set_preference(
        "extensions.firebug.defaultPanelName", "console"
    )
    profile.set_preference(
        "extensions.firebug.console.defaultPersist", "true"
    )
    profile.set_preference(
        "extensions.firebug.consoleFilterTypes", "error"
    )
    profile.set_preference("extensions.firebug.showFirstRunPage", False)
    profile.set_preference("extensions.firebug.cookies.enableSites", True)
    if device.user_agent:
        profile.set_preference("general.useragent.override", device.user_agent)
    profile.update_preferences()
    return profile


def get_firefox_options(device):
    """Function available if selenium version > 2.53.0"""
    from selenium.webdriver.firefox.options import Options as FirefoxOptions
    firefox_options = FirefoxOptions()
    firefox_options.add_argument("-no-remote")
    firefox_options.add_argument(f"-width {device.width}")
    firefox_options.add_argument(f"-height {device.height}")
    return firefox_options


PERFORMANCE_CAPABILITIES = (
    'goog:loggingPrefs',
    {
        "performance": "ALL",
        "server": "ALL",
        "client": "ALL",
        "driver": "ALL",
        "browser": "ALL"
    }
)


CERTS_CAPABILITIES = {
    'acceptSslCerts': True,
    'acceptInsecureCerts': True
}


class RemoteBrowser(webdriver.Remote):

    def __init__(self, *args, client, device, hub, proxy_server,
                 is_headless, browser_options=None, **kwargs):
        """
        Initialise a new Remote WebDriver instance
        with connection to Selenium Grid
        """
        capabilities = getattr(DesiredCapabilities, client.upper()).copy()

        if client == "chrome":
            options = get_chrome_options(
                device, proxy_server, is_headless, browser_options
            )
            if is_headless:
                options.accept_insecure_certs = True
        elif client == "firefox":
            options = get_firefox_options(device)
            kwargs['browser_profile'] = get_firefox_profile(device)
        elif client == "opera":
            options = get_opera_options(device)
            capabilities["browserName"] = "opera"
        if client != "safari":
            if proxy_server:
                proxy_capabilities = Proxy(
                    {
                        "httpProxy": proxy_server,
                        'sslProxy': proxy_server,
                        'noProxy': None,
                        "proxyType": ProxyType.MANUAL
                    }
                )
            else:
                proxy_capabilities = Proxy(
                    {
                        "proxyType": ProxyType.DIRECT
                    }
                )
            capabilities.update(proxy_capabilities.to_capabilities())

        options.set_capability(*PERFORMANCE_CAPABILITIES)
        options.set_capability('cloud:options', capabilities)
        kwargs['options'] = options
        remote_connection = RemoteConnection(hub, keep_alive=False)
        remote_connection.set_timeout(300)
        kwargs['command_executor'] = remote_connection
        super().__init__(*args, **kwargs)


class ChromeBrowser(webdriver.Chrome):

    def __init__(
            self,
            *args,
            device,
            proxy_server,
            is_headless,
            browser_options=None,
            **kwargs,
    ):
        """
        Initialise a new Chrome WebDriver instance.
        """
        capabilities = DesiredCapabilities.CHROME.copy()

        if proxy_server:
            proxy_capabilities = Proxy(
                {
                    "httpProxy": proxy_server,
                    'sslProxy': proxy_server,
                    'noProxy': None,
                    "proxyType": ProxyType.MANUAL
                }
            )
            capabilities.update(proxy_capabilities.to_capabilities())

        options = get_chrome_options(
            device, proxy_server, is_headless, browser_options,
        )
        if is_headless:
            options.accept_insecure_certs = True
        options.add_argument("--disable-dev-shm-usage")
        options.set_capability(*PERFORMANCE_CAPABILITIES)
        options.set_capability('cloud:options', capabilities)
        kwargs['options'] = options

        super().__init__(*args, **kwargs)


class OperaBrowser(webdriver.Chrome):

    def __init__(self, *args, device, **kwargs):
        """
        Initialise a new Opera WebDriver instance.
        """

        capabilities = DesiredCapabilities.CHROME.copy()
        options = get_opera_options(device)

        kwargs['desired_capabilities'] = capabilities
        kwargs['options'] = options

        super().__init__(*args, **kwargs)


class FirefoxBrowser(webdriver.Firefox):

    def __init__(self, *args, device, **kwargs):
        """
        Initialise a new Firefox WebDriver instance.
        """

        capabilities = DesiredCapabilities.FIREFOX.copy()

        profile = get_firefox_profile(device)
        options = get_firefox_options(device)
        options.set_capability(*PERFORMANCE_CAPABILITIES)
        kwargs['desired_capabilities'] = capabilities
        kwargs['firefox_profile'] = profile
        kwargs['options'] = options

        super().__init__(*args, **kwargs)



import pytest
from bs4 import BeautifulSoup

from promium.test_case import RequestTestCase


@pytest.mark.request
class TestMainPage(RequestTestCase):
    test_case_url = 'some url with test case'

    def test_google_search_block(self):
        page_text = self.get_response('https://prom.ua/ua/').data
        soup = BeautifulSoup(page_text, "html.parser")

        search_input = soup.find(attrs={'name': 'search_term'})
        self.soft_assert_true(
            search_input,
            'Не найдена строка поиска'
        )

        search_btn = soup.find(attrs={'data-qaid': 'search_btn'})
        self.soft_assert_equals(
            search_btn.text,
            'Знайти',
            'Не правильное название кнопки поиска'
        )




from promium.base import Element


class Checkbox(Element):

    def select(self):
        """Selects checkbox from any default state"""
        if not self.is_selected():
            self.click()

    def deselect(self):
        """Deselects checkbox from any default state"""
        if self.is_selected():
            self.click()



import os
import datetime
import logging
import urllib3
from collections import namedtuple

import pytest
import allure

from selenium.common.exceptions import WebDriverException
from promium.test_case import create_driver
from promium.common import upload_screenshot
from promium.device_config import CHROME_DESKTOP_1920_1080, DesktopDevice
from promium.logger import Logger
from promium.exceptions import BrowserConsoleException, PromiumException


log = logging.getLogger(__name__)

FailInfoSelenium = namedtuple('FailInfoSelenium', [
    'test_type', 'url', 'screenshot', 'test_case', 'run_command', 'txid'
])
FailInfoRequest = namedtuple('FailInfoRequest', [
    'test_type', 'url', 'test_case', 'status_code', 'run_command', 'txid'
])

DEFAULT_HEADERS = {
    "Accept-Encoding": 'gzip, deflate',
    "Accept": "*/*",
    "Connection": "keep-alive",
}


@pytest.hookimpl
def pytest_addoption(parser):
    parser.addoption(
        "--capturelog",
        dest="logger",
        default=None,
        action="store_true",
        help="Show log messages for each failed test"
    )
    parser.addoption(
        "--fail-debug-info",
        action="store_true",
        help="Show screenshot and test case urls for each failed test"
    )
    parser.addoption(
        "--duration-time",
        action="store_true",
        help="Show the very slow test"
    )
    parser.addoption(
        "--allure-report",
        action="store_true",
        help="Generate allure report"
    )
    parser.addoption(
        "--highlight",
        action="store_true",
        help="Highlighting elements"
    )
    parser.addoption(
        "--headless",
        action="store_true",
        help="Enable headless mode for Chrome browser only"
    )
    parser.addoption(
        "--chrome",
        action="store_true",
        help="Use chrome browser"
    )
    parser.addoption(
        "--firefox",
        action="store_true",
        help="Use firefox browser"
    )
    parser.addoption(
        "--opera",
        action="store_true",
        help="Use opera browser"
    )
    parser.addoption(
        "--repeat",
        action="store",
        default=1,
        type=int,
        metavar='repeat',
        help='Number of times to repeat each test. Mostly for debug purposes'
    )
    parser.addoption(
        "--check-console",
        action="store_true",
        help="Check browser console js and other errors"
    )
    parser.addoption(
        "-R",
        "--replace-headless",
        action="store",
        help=(
            """ When selenium tests running in Chrome Headless mode,
            the Headless browser can recognized as a bot,
            so we change the agent's username to a real one.
            By default user agent contains something like
            "HeadlessChrome/86.0.4240.75", but we need "Chrome/86.0.4240.75"
            You need to specify the Chrome version. """
        )
    )


@pytest.fixture()
def request_init(request):

    def _get_fail_debug(request):
        """Failed test report generator"""
        if hasattr(request.config, "get_failed_test_command"):
            run_command = request.config.get_failed_test_command(request)
        else:
            run_command = "Please, add run command in your fixture."

        session = request.cls.session
        if session and not hasattr(session, 'status_code'):
            session.status_code = None

        return FailInfoRequest(
            test_type='request',
            url=(session.url, [None])[0],
            test_case=(request.cls.test_case_url, [None])[0],
            status_code=(session.status_code, [None])[0],
            run_command=(run_command, [None])[0],
            txid=(request.cls.xrequestid, [None])[0],
        )

    session = urllib3.PoolManager(
        cert_reqs=False,
        timeout=15,
        headers=DEFAULT_HEADERS,
    )
    session.url = (
        'Use self.get_response(url) for request tests or '
        'util methods for api tests!'
    )
    request.cls.session = session
    request.config.get_fail_debug = _get_fail_debug
    request.config.assertion_errors = request.cls.assertion_errors = []
    yield session
    request.cls.xrequestid = None


@pytest.fixture()
def driver_init(request):
    def _get_screenshot_png():
        try:
            return driver.get_screenshot_as_png()
        except Exception as e:
            log.error(f'[PROMIUM] Have problem with make screenshot: {e}')
            return 'Error'

    def _get_fail_debug(request):
        """Failed test report generator"""
        driver = request.config.driver

        alerts = 0
        try:
            while driver.switch_to.alert:
                alert = driver.switch_to.alert
                print('Unexpected ALERT: %s\n' % alert.text)
                alerts += 1
                alert.dismiss()
        except Exception:
            if alerts != 0:
                print('')
        try:
            url = driver.current_url
        except Exception as e:
            url = str(e)
        screenshot = upload_screenshot(driver)

        if hasattr(request.config, "get_failed_test_command"):
            run_command = request.config.get_failed_test_command(request)
        else:
            run_command = "Please, add run command in your fixture."

        return FailInfoSelenium(
            test_type='selenium',
            url=(url, [None])[0],
            screenshot=(screenshot, [None])[0],
            test_case=(request.cls.test_case_url, [None])[0],
            run_command=(run_command, [None])[0],
            txid=(request.cls.xrequestid, [None])[0],
        )

    def _is_skipped_error(request, error):
        for msg in request.cls.excluded_browser_console_errors:
            if msg["msg"] in error and msg["comment"]:
                return True
        return False

    def _check_console_errors(request):
        driver = request.config
        if hasattr(driver, "console_errors"):
            if driver.console_errors:
                browser_console_errors = driver.console_errors
                if hasattr(request.cls, "excluded_browser_console_errors"):
                    if request.cls.excluded_browser_console_errors:
                        try:
                            return list(filter(
                                lambda err: not _is_skipped_error(
                                    request, err),
                                browser_console_errors
                            ))
                        except Exception as e:
                            raise PromiumException(
                                "Please check your excluded errors list. "
                                "Original exception is: %s" % e
                            )
                return browser_console_errors
        return []

    mark = request.node.get_closest_marker('device')
    device = mark.args[0] if mark else CHROME_DESKTOP_1920_1080
    replacement_version = request.config.getoption('--replace-headless')
    if hasattr(device, "user_agent") and replacement_version:
        if not device.user_agent and "chrome" in os.environ.get("SE_DRIVER"):
            user_agent = (
                f"Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 "
                f"(KHTML, like Gecko) "
                f"Chrome/{replacement_version} Safari/537.36"
            )
            device = DesktopDevice(1920, 1080, user_agent)

    request.cls.device = device
    if hasattr(request.config, "proxy_server"):
        proxy_server = request.config.proxy_server
    else:
        proxy_server = None
    browser_options = getattr(request.config, "browser_options", None)
    driver = create_driver(
        device,
        proxy_server=proxy_server,
        browser_options=browser_options,
    )
    driver.xrequestid = request.cls.xrequestid
    request.cls.driver = request.config.driver = driver
    request.config.get_fail_debug = _get_fail_debug
    request.config.get_screenshot_png = _get_screenshot_png
    request.config.check_console_errors = _check_console_errors
    request.config.assertion_errors = request.cls.assertion_errors = []
    request.config.console_errors = driver.console_errors = []
    yield driver
    driver.xrequestid = request.cls.xrequestid = None
    request.config.console_errors = driver.console_errors = []
    try:
        driver.quit()
    except WebDriverException as e:
        log.error(f"[PROMIUM] Original webdriver exception: {e}")


def pytest_generate_tests(metafunc):
    if metafunc.config.option.repeat > 1:
        metafunc.fixturenames.append('repeat')
        metafunc.parametrize('repeat', range(metafunc.config.option.repeat))


@pytest.fixture(autouse=True)
def logger(request):
    return logging.getLogger()


@pytest.mark.trylast
def pytest_runtest_call(item):
    if hasattr(item.config, "assertion_errors"):
        if item.config.assertion_errors:
            separator = '-' * 46
            assertion_errors_list = f"{separator}\n".join(
                item.config.assertion_errors
            )
            msg = f"\n{separator}\n{assertion_errors_list}{separator}\n"
            raise AssertionError(msg)

    if item.config.getoption("--check-console"):
        if hasattr(item.config, "check_console_errors"):
            browser_console_errors = item.config.check_console_errors(item)
            if browser_console_errors:
                msg = (
                    "Browser console js errors found:"
                    "\n{console_errors}\n".format(
                        console_errors="\n".join(
                            browser_console_errors
                        )
                    )
                )
                raise BrowserConsoleException(msg)


@pytest.mark.tryfirst
@pytest.mark.hookwrapper
def pytest_runtest_makereport(item, call):
    """pytest failed test report generator"""
    outcome = yield
    report = outcome.get_result()
    when = report.when
    item.config.path_to_test = f'{item.location[0]} -k {item.name}'

    if when == "call" and not report.passed:
        if item.config.getoption("--fail-debug-info"):
            fail_info = "Fail info not found."
            try:
                if hasattr(item.config, "get_fail_debug"):
                    fail_debug = item.config.get_fail_debug(item)
                    if fail_debug.test_type == 'selenium':
                        fail_info = (
                            (
                                "\nURL: {url}"
                                "\nTEST CASE: {test_case}"
                                "\nSCREENSHOT: {screenshot}"
                                "\nRUN_COMMAND: {run_command}"
                                "\nTXID: {txid}"
                            ).format(
                                url=fail_debug.url,
                                screenshot=fail_debug.screenshot,
                                test_case=fail_debug.test_case,
                                run_command=fail_debug.run_command,
                                txid=fail_debug.txid
                            )
                        )
                        if item.config.getoption("--allure-report"):
                            allure.attach(
                                item.config.get_screenshot_png(),
                                name='SCREENSHOT',
                                attachment_type=allure.attachment_type.PNG,
                            )

                    elif fail_debug.test_type == 'request':
                        fail_info = (
                            "\nURL: {url}"
                            "\nTEST_CASE: {test_case}"
                            "\nSTATUS_CODE: {status_code}"
                            "\nRUN_COMMAND: {run_command}"
                            "\nTXID: {txid}"
                        ).format(
                            url=fail_debug.url,
                            test_case=fail_debug.test_case,
                            status_code=fail_debug.status_code,
                            run_command=fail_debug.run_command,
                            txid=fail_debug.txid
                        )
                    if item.config.getoption("--allure-report"):
                        if fail_debug.test_case:
                            for url in fail_debug.test_case.split():
                                allure.dynamic.link(url=url, name='TestCase')
                        allure.attach(
                            fail_debug.url, name='URL',
                            attachment_type=allure.attachment_type.URI_LIST,
                        )
                        allure.attach(
                            f"""<button onclick="navigator.clipboard.writeText(
                            '{fail_debug.run_command}');
                            this.textContent='Copied';">Copy</button> <br>
                            <div style="font-family: monospace,serif;
                            font-size: 14px;">{fail_debug.run_command}</div>""",
                            name='RUN COMMAND',
                            attachment_type=allure.attachment_type.HTML,
                        )

            except WebDriverException as e:
                fail_info = (
                    "\nWebdriver instance must have crushed: {msg}".format(
                        msg=e.msg
                    )
                )
            finally:
                longrepr = getattr(report, 'longrepr', None)
                if hasattr(longrepr, 'addsection'):
                    longrepr.sections.insert(
                        0, ("Captured stdout %s" % when, fail_info, "-")
                    )

        if item.config.getoption("--duration-time"):
            duration = call.stop - call.start
            if duration > 120:
                report.sections.append((
                    "Captured stdout %s" % when,
                    ("\n\n!!!!! The very slow test. "
                     "Duration time is %s !!!!!\n\n")
                    % datetime.datetime.fromtimestamp(duration).strftime(
                        "%M min %S sec"
                    )
                ))


def pytest_configure(config):
    os.environ['PYTHONWARNINGS'] = (
        'ignore:An HTTPS request has been made, '
        'ignore:A true SSLContext object is not available, '
        'ignore:Unverified HTTPS request is being made'
    )

    if config.getoption('--headless'):
        os.environ['HEADLESS'] = 'Enabled'
    if config.getoption("--highlight"):
        os.environ['HIGHLIGHT'] = 'Enabled'
    if config.getoption("--capturelog"):
        config.pluginmanager.register(Logger(), "logger")
    if config.getoption("--check-console"):
        os.environ['CHECK_CONSOLE'] = 'Enabled'
    if not getattr(config, "lastfailed_file", None):
        config.lastfailed_file = "lastfailed"




import os
import sys
import subprocess
import shutil
import pytest


class EnvironmentException(Exception):
    pass


def run_command(command):
    print(f'Running command: {command}')
    process = subprocess.run(
        command,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )

    if process.returncode != 0:
        print(process.stdout.decode())
        print(process.stderr.decode())
        raise EnvironmentException(f'Failed to run command: {command}')

    return process


@pytest.fixture
def docs_fixture(scope='function'):
    with open('version', 'w') as version_file:
        version_file.write('13.666.2')

    yield

    os.chdir('/work')
    if os.path.exists('doc/_build'):
        shutil.rmtree('doc/_build', ignore_errors=True)
    if os.path.exists('version'):
        os.remove('version')
    if os.path.exists('dist'):
        shutil.rmtree('dist', ignore_errors=True)
    if os.path.exists('Promium.egg-info'):
        shutil.rmtree('Promium.egg-info', ignore_errors=True)


class TestEnvironment():

    @pytest.mark.env
    @pytest.mark.skip('pass')
    def test_check_doc_command(self, docs_fixture):
        run_command(['sphinx-build', '-b', 'html', 'doc', 'public'])
        assert os.path.exists('public/index.html'), 'Not Create doc files'

    @pytest.mark.env
    def test_check_publish(self, docs_fixture):
        run_command(['python', 'setup.py', 'sdist'])
        run_command(['python', '-m', 'twine', 'check', 'dist/*'])




import os

import pytest

from promium.device_config import CHROME_DESKTOP_1024_768, CustomDevice
from promium.support.settings import (
    BASE_CAPABILITIES,
    get_download_path,
    SettingsDrivers,
    SettingsChrome,
    SettingsOpera,
    SettingsEDGE,
    SettingsIE,
    SettingsFirefox,
    Base,
)


@pytest.mark.unit
@pytest.mark.parametrize('driver, ex_settings', [
    ('chrome', SettingsChrome),
    ('opera', SettingsOpera),
    ('firefox', SettingsFirefox),
    ('edge', SettingsEDGE),
    ('ie', SettingsIE),
])
def test_get_driver_settings(driver, ex_settings):
    assert isinstance(SettingsDrivers().get(driver), ex_settings)


@pytest.mark.unit
def test_get_driver_settings_negative():
    with pytest.raises(ValueError):
        SettingsDrivers().get('chromium')


@pytest.mark.unit
def test_get_download_path():
    assert get_download_path() == '/tmp'


@pytest.mark.unit
def test_get_download_path_with_env():
    os.environ.setdefault('DOWNLOAD_PATH', '/xxx/123')
    assert get_download_path() == '/xxx/123'


@pytest.mark.unit
def test_driver_capabilities():
    assert isinstance(Base.driver_capabilities, dict)
    assert not Base.driver_capabilities


@pytest.mark.unit
def test_default_capabilities():
    assert isinstance(Base.default_capabilities, dict)
    assert Base.default_capabilities == BASE_CAPABILITIES


@pytest.mark.unit
def test_user_capabilities():
    assert isinstance(Base.user_capabilities, dict)
    assert not Base.user_capabilities


@pytest.mark.unit
def test_binary_path():
    assert isinstance(Base.binary_path, str)
    assert not Base.binary_path


@pytest.mark.unit
def test_device():
    assert isinstance(Base.device, tuple)
    assert Base.device == CHROME_DESKTOP_1024_768


@pytest.mark.unit
def test_default_options():
    assert isinstance(Base.default_options, list)
    assert not Base.default_options


@pytest.mark.unit
def test_user_options():
    assert isinstance(Base.user_options, list)
    assert not Base.user_options


@pytest.mark.unit
def test_default_experimental_options():
    assert isinstance(Base.default_exp_options, dict)
    assert not Base.default_exp_options


@pytest.mark.unit
def test_user_experimental_options():
    assert isinstance(Base.user_exp_options, dict)
    assert not Base.user_exp_options


@pytest.mark.unit
def test_default_preferences():
    assert isinstance(Base.default_preferences, dict)
    assert not Base.default_preferences


@pytest.mark.unit
def test_user_preferences():
    assert isinstance(Base.user_preferences, dict)
    assert not Base.user_preferences


@pytest.mark.unit
def test_driver_options():
    with pytest.raises(NotImplementedError):
        Base.driver_options()


@pytest.mark.unit
def test_options():
    assert Base.options() == []


@pytest.mark.unit
def test_options_with_default_values():
    Base.default_options = ['x']
    assert Base.options() == ['x']


@pytest.mark.unit
def test_options_with_user_values():
    Base.user_options = ['x']
    assert Base.options() == ['x']


@pytest.mark.unit
def test_options_with_user_and_default_values():
    Base.user_options = ['x1']
    Base.default_options = ['x2']
    assert Base.options() == ['x1']


@pytest.mark.unit
def test_experimental_options():
    assert Base.experimental_options() == {}


@pytest.mark.unit
def test_experimental_options_with_default_values():
    Base.default_exp_options = {'key1': 1}
    assert Base.experimental_options() == {'key1': 1}


@pytest.mark.unit
def test_experimental_options_with_user_values():
    Base.user_exp_options = {'key2': 2}
    assert Base.experimental_options() == {'key2': 2}


@pytest.mark.unit
def test_experimental_options_with_user_and_default_values():
    Base.user_exp_options = {'key2': 2}
    Base.default_exp_options = {'key1': 1}
    assert Base.experimental_options() == {'key2': 2}


@pytest.mark.unit
def test_capabilities():
    assert Base.capabilities() == BASE_CAPABILITIES


@pytest.mark.unit
def test_capabilities_with_default_values():
    Base.default_capabilities = {'key1': 1}
    assert Base.capabilities() == {'key1': 1}


@pytest.mark.unit
def test_capabilities_with_user_values():
    Base.user_capabilities = {'key2': 2}
    assert Base.capabilities() == {'key2': 2}


@pytest.mark.unit
def test_capabilities_with_user_and_default_values():
    Base.user_capabilities = {'key3': 3}
    assert Base.default_capabilities
    assert Base.capabilities() == {'key3': 3}


@pytest.mark.unit
def test_preference():
    assert Base.preference() == {}


@pytest.mark.unit
def test_preference_with_default_values():
    Base.default_preferences = {'key1': 1}
    assert Base.preference() == {'key1': 1}


@pytest.mark.unit
def test_preference_with_user_values():
    Base.user_preferences = {'key2': 2}
    assert Base.preference() == {'key2': 2}


@pytest.mark.unit
def test_preference_with_user_and_default_values():
    Base.user_preferences = {'key1': 1}
    Base.default_preferences = {'key2': 2}
    assert Base.preference() == {'key1': 1}


@pytest.mark.unit
def test_set_custom_device():
    Base.set_custom_device(
        width=666, height=13, user_agent='user agent xxx'
    )
    assert Base.device == CustomDevice(
        width=666, height=13, user_agent='user agent xxx'
    )


@pytest.mark.unit
def test_set_custom_device_without_user_agent():
    Base.set_custom_device(width=123, height=321, user_agent='')
    assert Base.device == CustomDevice(width=123, height=321, user_agent='')


@pytest.mark.unit
def test_set_options():
    Base.user_options = []
    Base.set_options('xxx', '123')
    assert Base.user_options == ['xxx', '123']


@pytest.mark.unit
def test_add_option():
    assert 'xxx' not in Base.default_options
    Base.add_option('xxx')
    assert 'xxx' in Base.default_options


@pytest.mark.unit
def test_delete_option():
    Base.default_options = ['xxx']
    Base.delete_option('xxx')
    assert 'xxx' not in Base.default_options


@pytest.mark.unit
def test_set_windows_size():
    Base.device = CustomDevice(width=123, height=321, user_agent='xxx')
    Base.set_windows_size()
    ex_options = [
        '--user-agent={}'.format(Base.device.user_agent),
        '--window-size={},{}'.format(Base.device.width, Base.device.height)
    ]
    for opt in ex_options:
        assert opt in Base.default_options


@pytest.mark.unit
@pytest.mark.parametrize('params', [(123, 321, ''), (321, 123, None)])
def test_set_windows_size_without_user_agent(params):
    Base.device = CustomDevice(*params)
    Base.set_windows_size()
    ex_option = '--window-size={},{}'.format(
        Base.device.width, Base.device.height
    )
    assert ex_option in Base.default_options


@pytest.mark.unit
def test_set_windows_size_with_device():
    device = CustomDevice(width=123, height=321, user_agent='user xxx')
    Base.set_windows_size(device)
    ex_options = [
        '--user-agent={}'.format(device.user_agent),
        '--window-size={},{}'.format(device.width, device.height)
    ]
    for opt in ex_options:
        assert opt in Base.default_options


@pytest.mark.unit
def test_get_options():
    with pytest.raises(NotImplementedError):
        Base.get_options()


@pytest.mark.unit
def test_get_capabilities():
    Base.default_capabilities = BASE_CAPABILITIES
    Base.user_capabilities = {}
    assert Base.get_capabilities() == BASE_CAPABILITIES


@pytest.mark.unit
def test_get_capabilities_with_user_values():
    Base.default_capabilities = BASE_CAPABILITIES
    Base.user_capabilities = {'key1': 1}
    assert Base.get_capabilities() == {'key1': 1}


@pytest.mark.unit
def test_base_set_preferences():
    Base.default_preferences = {}
    Base.user_preferences = {}
    Base.set_preferences(key1=1, key2=2)
    assert Base.user_preferences == {'key1': 1, 'key2': 2}


@pytest.mark.unit
def test_base_update_preferences():
    Base.default_preferences = {}
    Base.user_preferences = {}
    Base.update_preferences(key1=1, key2=2)
    assert Base.default_preferences == {'key1': 1, 'key2': 2}


@pytest.mark.unit
def test_base_delete_preferences():
    Base.default_preferences = {'key1': 1}
    Base.delete_preference('key1')
    assert Base.default_preferences == {}


@pytest.mark.unit
def test_get_preferences():
    assert Base.get_preferences() is None



import base64
import os
import datetime

import allure
import urllib3
import json
import time
import logging

from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.action_chains import ActionChains

from promium.waits import (
    wait_for_animation, enable_jquery, wait_until_new_window_is_opened
)


log = logging.getLogger(__name__)


http = urllib3.PoolManager(
    cert_reqs=False,
    timeout=5,
)


SCREENSHOT_SERVER_HOST = os.environ.get("SCREENSHOT_SERVER_HOST")
SCREENSHOT_SERVER_LOGIN = os.environ.get("SCREENSHOT_SERVER_LOGIN")
SCREENSHOT_SERVER_PASSWORD = os.environ.get("SCREENSHOT_SERVER_PASSWORD")


def scroll_to_bottom(driver):
    """Scrolls down page"""
    enable_jquery(driver)
    driver.execute_script('jQuery("img.img-ondemand").trigger("appear");')
    driver.execute_script(
        """
        var f = function(old_height) {
            var height = $$(document).height();
            if (height == old_height) return;
            $$('html, body').animate({scrollTop:height}, 'slow', null,
            setTimeout(function() {f(height)}, 1000)
            );
        }
        f();
        """
    )
    wait_for_animation(driver)


def scroll_to_top(driver):
    """Scrolls to the top of the page"""
    enable_jquery(driver)
    driver.execute_script(
        """jQuery('html, body').animate({scrollTop: 0 }, 'slow', null);"""
    )
    wait_for_animation(driver)


def scroll_to_bottom_in_block(driver, element_class):
    """Scrolls to the bottom of the current block"""
    enable_jquery(driver)
    script = """
        var elem = '.'.concat(arguments[0]);
        $(elem).animate({scrollTop: $(elem).prop('scrollHeight')}, 1000);
    """
    driver.execute_script(script, element_class)
    wait_for_animation(driver)


def scroll_to_element(driver, element, base_element=None):
    """
    use base_element if you need for example scroll into popup,
    base_element must be a popup locator.
    """
    enable_jquery(driver)
    if base_element is None:
        base_element = 'html, body'
    script = """
        var elem = arguments[0];
        var base = arguments[1];
        var relativeOffset = (
            jQuery(elem).offset().top - jQuery(base).offset().top
        );
        jQuery(base).animate({
            scrollTop: relativeOffset
            }, 'slow', null
        );
             """
    driver.execute_script(script, element, base_element)
    wait_for_animation(driver)


def scroll_with_offset(driver, element, with_offset=0):
    """
    Adjusting the final scroll position by adding or subtracting a
    specified number of pixels
    """
    enable_jquery(driver)
    script = """
        var elem = arguments[0];
        jQuery('html, body').animate({
            scrollTop: jQuery(elem).offset().top + arguments[1]
            }, 'fast', null
        );
        """
    driver.execute_script(script, element, with_offset)
    wait_for_animation(driver)


def scroll_into_end(driver):
    """Scroll block into end page"""
    enable_jquery(driver)
    driver.execute_script(
        'window.scrollTo(0, document.body.scrollHeight);'
    )
    wait_for_animation(driver)


def open_link_in_new_tab(driver, url=None):
    """
    Opening URL in new tab and switches to recently opened window
    If URL is None - an empty window will open
    """
    main_window = driver.current_window_handle
    driver.execute_script(
        f'''window.open("{
        "about:blank" if url is None else url
        }","_blank");'''
    )
    new_window = wait_until_new_window_is_opened(driver, main_window)
    switch_to_window(driver, new_window)


def switch_to_window(driver, window_handle):
    """Switches to window"""
    driver.switch_to.window(window_handle)


def get_screenshot_path(name, with_date=True):
    now = datetime.datetime.now().strftime('%d_%H_%M_%S_%f')
    screenshot_name = f"{name}_{now}.png" if with_date else f"{name}.png"
    screenshots_folder = "/tmp"
    if not os.path.exists(screenshots_folder):
        os.makedirs(screenshots_folder)
    screenshot_path = os.path.join(screenshots_folder, screenshot_name)
    return screenshot_path, screenshot_name


def if_server_have_problem():
    try:
        r = http.request(
            url=f"https://{SCREENSHOT_SERVER_HOST}",
            method='GET',
            headers={
                'Accept-Encoding': 'gzip, deflate',
                'Accept': '*/*',
                'Connection': 'keep-alive',
            },
        )
        assert r.status <= 400, f'Have problem {r.status}: {r.reason}'
        return False
    except Exception as e:
        return str(e)


def _upload_to_server(screenshot_path, screenshot_name):

    status_server = if_server_have_problem()
    if status_server:
        return status_server

    def read_in_chunks(img, block_size=1024, chunks=-1):
        """
        Lazy function (generator) to read a file piece by piece.
        Default chunk size: 1k.
        """
        while chunks:
            data = img.read(block_size)
            if not data:
                break
            yield data
            chunks -= 1

    screenshot_url = f"https://{SCREENSHOT_SERVER_HOST}/{screenshot_name}"
    r = http.request(
        url=screenshot_url,
        method='PUT',
        headers={
            'Accept-Encoding': 'gzip, deflate',
            'Accept': '*/*',
            'Connection': 'keep-alive',
            'Content-Type': 'application/json',
            'Authorization': 'Basic ' + base64.b64encode(
                f'{SCREENSHOT_SERVER_LOGIN}:{SCREENSHOT_SERVER_PASSWORD}'
                ''.encode()
            ).decode()
        },
        body=read_in_chunks(open(screenshot_path, 'rb')),
    )
    os.remove(screenshot_path)
    if r.status != 201:
        return (
            f"Screenshot not uploaded to server. "
            f"Status code: {r.status_code}, reason: {r.reason}"
        )
    return screenshot_url


def upload_screenshot(driver, path_name="screenshot", path_with_date=True):
    try:
        screenshot_path, screenshot_name = get_screenshot_path(
            name=path_name,
            with_date=path_with_date
        )
        driver.save_screenshot(screenshot_path)

        if SCREENSHOT_SERVER_HOST:
            return _upload_to_server(screenshot_path, screenshot_name)
        return f"file://{screenshot_path}"

    except Exception as e:
        return f'No screenshot was captured due to exception {repr(e)}'


def control_plus_key(driver, key):
    """Imitations press CONTROL key + any key"""
    (
        ActionChains(driver)
        .key_down(Keys.CONTROL)
        .send_keys(key)
        .key_up(Keys.CONTROL)
        .perform()
    )


def set_local_storage(driver, key, value):
    """Sets value in browsers local storage"""
    driver.execute_script(f"localStorage.setItem('{key}', '{value}')")


def delete_cookie_item(driver, name):
    """Deleting cookie on current page"""
    driver.delete_cookie(name)
    driver.refresh()


def delete_element_from_dom(driver, element):
    """Removes a specific DOM element from the webpage"""
    enable_jquery(driver)
    driver.execute_script(f"""
        var element = document.querySelector("{element}");
        if (element)
        element.parentNode.removeChild(element);
    """)


def find_network_log(
        driver, find_mask, find_status=200, timeout=5, find_type=None
):
    """ find xrh response and request data from browser network logs
    :param driver: selenium driver
    :param find_mask: find word in request url
    :param find_status: find response with this status
    :param timeout: tries
    :param find_type: check type request in network (example find_type='Fetch')
    :return: [{response, data, body}, {response, data, body}] or []
    """
    def get_log(request_mask):
        found_requests = []
        for performance_log in driver.get_log('performance'):
            # only response logs
            if (
                request_mask in performance_log['message'] and
                'responsereceived' in performance_log['message'].lower()
            ):
                json_log = json.loads(performance_log['message'])['message']
                if not json_log['params'].get('response'):
                    continue
                response = json_log['params']['response']
                if (
                    response['status'] == find_status and (
                        not find_type or json_log['params']['type'] == find_type
                    )
                ):
                    found_requests.append(json_log)
        return found_requests

    while timeout >= 0:
        xhr_log = get_log(find_mask)
        if xhr_log:
            break
        time.sleep(0.5)
        timeout -= 1

    result_all_logs = []
    for network_log in xhr_log:
        requestId = network_log['params']['requestId']  # noqa: N806
        response = network_log['params']['response']
        try:
            request_data = driver.execute_cdp_cmd(
                'Network.getRequestPostData', {'requestId': requestId}
            )
            request_data = json.loads(request_data['postData'])
        except Exception:
            request_data = None

        try:
            response_body = driver.execute_cdp_cmd(
                'Network.getResponseBody', {'requestId': requestId}
            )
            response_body = json.loads(response_body['body'])
        except Exception:
            response_body = None

        log.info(
            f"[XHR Logs]: find by mask: '{find_mask}' -> {response.get('url')} "
            f"{response.get('status')} - {response.get('statusText')}"
        )
        result_all_logs.append({
            'response': {
                'url': response.get('url'),
                'status': response.get('status'),
                'statusText': response.get('statusText'),
            },
            'data': request_data,
            'body': response_body,
        })

    allure.attach(
        name=f"[XHR log] '{find_mask}', status:{find_status}, type:{find_type}",
        body=json.dumps(result_all_logs, indent=4, ensure_ascii=False),
        attachment_type=allure.attachment_type.JSON,
    )
    return result_all_logs




from tests.config import PROJECT_HOST, PROJECT_PORT, WIKI_HOST


def collect_url(path, sub_domain=''):
    """
    Collects url
    :param str path:
    :param str sub_domain:
    :return str url
    """
    return f'https://{sub_domain}{PROJECT_HOST}{PROJECT_PORT}/{path}'


def case_url(case_id):
    """
    Returns check-list page by id
    :param int case_id: Page id
    :return str url
    """
    return f'https://{WIKI_HOST}{case_id}'



from promium.base import Page, Block, Element
from promium.elements.checkbox import Checkbox
from promium.elements.radio_button import RadioButton
from promium.elements.upload_button import UploadButton
from promium.elements.input_field import InputField
from promium.elements.link import Link
from promium.elements.drop_downs import DropDown
from promium.elements.textarea_field import TextareaField
from promium.elements.multiselect_listbox import MultiSelectListBox
from promium.elements.frame import Frame


__all__ = (
    'Page',
    'Block',
    'Element',
    'Checkbox',
    'RadioButton',
    'UploadButton',
    'InputField',
    'TextareaField',
    'Link',
    'DropDown',
    'MultiSelectListBox',
    'Frame',
)




import codecs
from setuptools import setup, find_packages


REQUIREMENTS = [
    'six>=1.10.0',
    'json_checker>=1.2.1',
    'urllib3>=2.0.2',
    'py>=1.5.2',
    'pytest>=5.0.0',
    'pytest_rerunfailures>=4.1.0',
    'pytest-forked>=0.2',
    'pytest-instafail>=0.3.0',
    'deepdiff>=5.7.0',
    'selenium>=4.10.0',
]


setup(
    name='Promium',
    version='3.6.6',
    install_requires=REQUIREMENTS,
    author='Denis Korytkin, Nataliia Guieva, '
           'Roman Zaporozhets, Vladimir Kritov, '
           'Oleh Dykusha',
    project_urls={
        'Home page': 'https://none',
        'Documentation': 'https://none',
    },
    description='Selenium wrapper for testing Web UI',
    long_description=codecs.open('README.rst', 'r', 'utf-8').read(),
    keywords=['Testing UI', 'Selenium', 'PageObject', 'Selenium wrapper'],
    platforms=['linux'],
    packages=find_packages(),
    entry_points={'pytest11': ['promium = promium.plugin']},
    python_requires='>=3.10, <4',
    classifiers=[
        'Programming Language :: Python :: 3.13',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Testing',
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
    ]
)



import time

import pytest
from mock import patch, MagicMock

from promium.base import Element
from promium.exceptions import PromiumException, PromiumTimeout
from promium.waits import (
    wait,
    wait_until,
    wait_until_not,
    wait_for_status_code,
    wait_until_new_window_is_opened,
    wait_url_contains,
    wait_for_alert,
)
from promium.expected_conditions import is_present
from selenium.common.exceptions import TimeoutException


FAKE_LOCATOR = ('', '')


@pytest.fixture()
def driver():
    return MagicMock(name='driver', **{
        'refresh.return_value': 'refresh OK',
        'session_id': '666',
        'window_handles': ['old_window', 'new_window_0', 'new_window_1'],
        'current_url': 'http://www.example.com/testing',
    })


def return_alert(status=None):
    return MagicMock(name='driver', **{
        'switch_to.alert': status,
    })


@pytest.mark.unit
def test_base_wait(driver):
    from selenium.webdriver.support.ui import WebDriverWait
    w = wait(driver)
    assert isinstance(w, WebDriverWait)
    assert hasattr(w, 'until')
    assert hasattr(w, 'until_not')


@pytest.mark.unit
@patch.object(Element, '_find_elements', return_value=True)
@pytest.mark.parametrize('seconds, msg', [
    (.01, None),
    (0, ''),
    (.2, 'test error message')
])
def test_wait_until(mock_element, seconds, msg, driver):
    element = Element(*FAKE_LOCATOR)
    element.driver = driver
    start_time = time.time()
    wait_until(driver, is_present(element), seconds, msg)
    end_time = int(time.time() - start_time)
    mock_element.assert_called_once()
    assert end_time == 0, 'Not correct wait time duration'


@pytest.mark.unit
@patch.object(Element, '_find_elements', return_value=False)
@pytest.mark.parametrize('seconds, msg, ex_result', [
    (1, None, (1 * 2) + 1),
    (0, '', 1),
    (5, 'test error message', (5 * 2) + 1)
])
def test_wait_until_negative(mock_element, seconds, msg, ex_result, driver):
    element = Element(*FAKE_LOCATOR)
    element.driver = driver
    start_time = time.time()
    with pytest.raises(PromiumTimeout):
        wait_until(driver, is_present(element), seconds, msg)
    end_time = int(time.time() - start_time)
    mock_element.assert_called()
    assert mock_element.call_count == ex_result
    assert end_time == seconds, 'Not correct wait time duration'


@pytest.mark.unit
@pytest.mark.parametrize('var, ex, seconds', [
    ([True], 1, 0),
    ([False, True], 2, 0),
    ([False, False, True], 3, 1),
    ([False] * 4 + [True], 5, 2),
])
def test_wait_until_count(var, ex, seconds, driver):
    with patch.object(Element, '_find_elements', side_effect=var) as el:
        element = Element(*FAKE_LOCATOR)
        element.driver = driver
        start_time = time.time()
        wait_until(driver, is_present(element))
        end_time = int(time.time() - start_time)
        el.assert_called()
        assert el.call_count == ex
        assert end_time == seconds, 'Not correct wait time duration'


@pytest.mark.unit
@pytest.mark.parametrize('var, ex, seconds', [
    ([False], 1, 0),
    ([True, False], 2, 0),
    ([True, True, False], 3, 1),
    ([True] * 4 + [False], 5, 2),
])
def test_wait_until_not_positive(var, ex, seconds, driver):
    with patch.object(Element, '_find_elements', side_effect=var) as el:
        element = Element(*FAKE_LOCATOR)
        element.driver = driver
        start_time = time.time()
        wait_until_not(driver, is_present(element))
        end_time = int(time.time() - start_time)
        el.assert_called()
        assert el.call_count == ex
        assert end_time == seconds, 'Not correct wait time duration'


@pytest.mark.unit
@patch.object(Element, '_find_elements', return_value=True)
@pytest.mark.parametrize('ex, seconds', [(1, 0), (3, 1), (11, 5)])
def test_wait_until_not_negative(mock_element, ex, seconds, driver):
    element = Element(*FAKE_LOCATOR)
    element.driver = driver
    start_time = time.time()
    with pytest.raises(PromiumException):
        wait_until_not(driver, is_present(element), seconds)
    end_time = int(time.time() - start_time)
    mock_element.assert_called()
    assert mock_element.call_count == ex
    assert end_time == seconds, 'Not correct wait time duration'


@pytest.mark.unit
@pytest.mark.parametrize('var, ex, ex_sec', [
    ([True], 1, 0),
    ([False, True], 2, 0),
    ([False, False, True], 3, 1),
    ([False, False] * 3 + [True], 7, 3),
])
def test_wait_until_with_reload(var, ex, ex_sec, driver):
    with patch.object(Element, '_find_elements', side_effect=var) as el:
        element = Element(*FAKE_LOCATOR)
        element.driver = driver
        start_time = time.time()
        wait_until(
            driver=driver,
            expression=is_present(element, with_refresh=True),
            seconds=10
        )
        end_time = int(time.time() - start_time)
        el.assert_called()
        assert el.call_count == ex
        assert end_time == ex_sec, 'Not correct wait time duration'
        driver.refresh.assert_called()
        assert driver.refresh.call_count == ex


@pytest.mark.unit
@patch.object(Element, '_find_elements', return_value=False)
@pytest.mark.parametrize('ex, sec', [(1, 0), (3, 1), (11, 5)])
def test_wait_until_with_reload_negative(mock_element, ex, sec, driver):
    element = Element(*FAKE_LOCATOR)
    element.driver = driver
    start_time = time.time()
    with pytest.raises(PromiumTimeout):
        wait_until(
            driver=driver,
            expression=is_present(element, with_refresh=True),
            seconds=sec
        )
    end_time = int(time.time() - start_time)
    mock_element.assert_called()
    assert end_time == sec, 'Not correct wait time duration'
    driver.refresh.assert_called()
    assert driver.refresh.call_count == ex


@pytest.mark.unit
def test_wait_for_alert():
    start_time = time.time()
    wait_status = wait_for_alert(
        driver=return_alert(True),
        seconds=1
    )
    end_time = int(time.time() - start_time)
    assert wait_status is True, 'Not correct wait return status'
    assert end_time < 1, 'Not correct wait time duration'


@pytest.mark.unit
@pytest.mark.parametrize('sec', [0, 2, 3])
def test_wait_for_alert_negative(sec):
    start_time = time.time()
    with pytest.raises(TimeoutException):
        wait_for_alert(
            driver=return_alert(False),
            seconds=sec
        )
    end_time = int(time.time() - start_time)
    assert end_time == sec, 'Not correct wait time duration'


@pytest.mark.unit
@pytest.mark.parametrize('tries', [1, 2, 10, 15])
def test_wait_for_status_code(tries):
    start_time = time.time()
    wait_for_status_code('http://google.com/page?id=666', 404, tries)
    end_time = int(time.time() - start_time)
    assert end_time == 0


@pytest.mark.unit
@pytest.mark.parametrize('tries', [1, 3])
def test_wait_for_status_code_negative(tries):
    start_time = time.time()
    with pytest.raises(PromiumException):
        wait_for_status_code('http://google.com/page?id=666', 200, tries)
    end_time = int(time.time() - start_time)
    assert end_time == tries


@pytest.mark.unit
@pytest.mark.parametrize('index, window_handle', [
    (0, 'new_window_0'),
    (1, 'new_window_1'),
])
def test_wait_until_new_window_is_opened(index, window_handle, driver):
    element = Element(*FAKE_LOCATOR)
    element.driver = driver
    new_window = wait_until_new_window_is_opened(driver, 'old_window', index)
    assert new_window == window_handle


@pytest.mark.unit
def test_wait_until_new_window_is_opened_negative(driver):
    element = Element(*FAKE_LOCATOR)
    element.driver = driver
    try:
        wait_until_new_window_is_opened(driver, 'some_unreal_window_name')
    except ValueError:
        pass


@pytest.mark.unit
def test_wait_url_contains1(driver):
    element = Element(*FAKE_LOCATOR)
    element.driver = driver
    start_time = time.time()
    wait_url_contains(driver, 'example.com', timeout=1)
    end_time = int(time.time() - start_time)
    assert end_time == 0, 'Not correct wait time duration'


@pytest.mark.unit
@pytest.mark.parametrize('sec', [1, 2, 4])
def test_wait_url_contains_negative(driver, sec):
    element = Element(*FAKE_LOCATOR)
    element.driver = driver
    start_time = time.time()
    with pytest.raises(PromiumTimeout):
        wait_url_contains(driver, 'uncorrect url', timeout=sec)
    end_time = int(time.time() - start_time)
    assert end_time == sec, 'Not correct wait time duration'



from selenium.webdriver.common.keys import Keys

from promium.base import Element
from promium.logger import log
from promium.exceptions import PromiumException


class InputField(Element):

    @property
    def value(self):
        """Returns attribute value"""
        return self.get_attribute("value")

    @property
    def placeholder(self):
        """Returns attribute placeholder"""
        return self.get_attribute("placeholder")

    def clear(self):
        """Clears the text if it's a text entry element."""
        return self.lookup().clear()

    def set_value(self, value):
        """Sets data by the given value"""
        self.driver.execute_script(
            f'arguments[0].value = "{value}"', self.lookup()
        )

    def send_keys(self, *value):
        """Sends keys by the given value"""
        if None in value:
            raise PromiumException(
                f"One of the values is None: {value}. Must be a string."
            )
        return self.lookup().send_keys(*value)

    def _clear_field_with_keyboard(self):
        """Clear the field using keyboard keys"""
        self.send_keys(f'{Keys.CONTROL}a')
        self.send_keys(Keys.BACKSPACE)

    def clear_and_fill(self, text):
        """
        Clear the field using keyboard keys and fill it with the given text
        """
        expected_text = str(text).lower()
        self._clear_field_with_keyboard()
        self.send_keys(text)
        if self.value.lower() == expected_text:
            return expected_text
        log.warning(
            f"Different text input value after clear with keyboard and fill. "
            f"Expected - {expected_text}, current - {self.value}"
        )

    def fill_field(self, text):
        """Clear element value if exists and fills it with the given text"""
        expected_text = str(text).lower()
        if self.value:
            self.clear()
        self.send_keys(text)
        if self.value.lower() == expected_text:
            return expected_text
        log.warning(
            f"Different text input value after fill. "
            f"Expected - {expected_text}, current - {self.value}"
        )



import pytest
from mock import Mock

from promium.base import Element
from promium.assertions import BaseSoftAssertion, RequestSoftAssertion


FALSE_PARAMS = [False, 0, '', [], {}, ()]
TRUE_PARAMS = [True, 1, 'sada', [1], {'k': 'test'}, (1, 2)]
EQUALS_PARAMS = [
    (1, 1), (0, 0), ('t', 't'), ('u', u'u'),
    (False, False), (True, 1), (False, 0),
]
EQUALS_PARAMS_EXTENDED = EQUALS_PARAMS + [
    ([1, 's', []], [1, 's', []]),
    ({'k': 1}, {'k': 1})
]
NOT_EQUALS_PARAMS = [
    (1, 0), (0, 'www'), ('t', 'test'), ('u', u'unicode'), (False, True),
    ([1, 's', []], [1, 's', ['test']]), ({'k': []}, {'k': 1})
]
CONTAIN_PARAMS = [
    (1, [1]), (2, {1, 2}), (12, (6, 12)), (156, list(range(120, 1000))),
    ('str', ['str']), ('str', [u'str']), ('str', {'str'}),
    ('str', ('str',)), ('str', {'str': 1, 'key': 2}),
    ('string', 'test string'), ('string', u'test string'), ('str', 'teststr')
]
NOT_CONTAIN_PARAMS = [
    (1, [0]), (2, {1, 6}), (12, (6, 1)), (156, list(range(120))),
    ('str1', ['str']), ('string', [u'str1']), ('str', {'str1'}),
    ('str', ('tuple',)), ('str', {'str_key': 1, 'key': 2}),
    ('string', 'test '), ('string', u'test')
]
LESS_PARAMS = [(0, 1), ('sss', 'ssss'), (False, True)]
GREATER_PARAMS = [(1, 0), ('sss', 's'), (True, False)]
NOT_LESS_EQUALS_PARAMS = GREATER_PARAMS + [
    ('test', 't'), (u'unicode', 'u'), (True, False)
]
NOT_GREATER_EQUALS_PARAMS = LESS_PARAMS + [
    (0, 1), ('t', 'test'), ('u', u'unicode'), (False, True)
]
INSTANCE_PARAMS = [
    ('', str), (1, int), ([], list), ({'k': False}, dict), ((1, 2), tuple),
    ([1], list), (True, bool), ('1', str), ('666', (int, str))
]
NOT_INSTANCE_PARAMS = [
    (1, str), ('1', int), (1, bool), (0, bool), ({'k': 1}, str),
    ({'k': 1}, int), ([], (set, tuple))
]
SPACES_PARAMS = [
    ('s ', 's'), ('\n\t \r', ''), ('\v|\f |\n', '| |'), (' |    |   ', '| |')
]


class MockElement(object):

    def __init__(self, some_dict):
        self.some_dict = some_dict

    def get_attribute(self, name):
        return self.some_dict.get(name)


@pytest.mark.unit
@pytest.mark.parametrize('expr', TRUE_PARAMS)
def test_soft_assert_true_valid_data(expr):
    soft = BaseSoftAssertion()
    soft.soft_assert_true(expr)
    assert not soft.assertion_errors


@pytest.mark.unit
@pytest.mark.parametrize('expr', FALSE_PARAMS)
def test_soft_assert_true_not_valid_data(expr):
    soft = BaseSoftAssertion()
    soft.assertion_errors = []
    soft.soft_assert_true(expr)
    assert soft.assertion_errors
    assert len(soft.assertion_errors) == 1


@pytest.mark.unit
@pytest.mark.parametrize('expr', FALSE_PARAMS)
def test_soft_assert_false_valid_data(expr):
    soft = BaseSoftAssertion()
    soft.soft_assert_false(expr)
    assert not soft.assertion_errors


@pytest.mark.unit
@pytest.mark.parametrize('expr', TRUE_PARAMS)
def test_soft_assert_false_not_valid_data(expr):
    soft = BaseSoftAssertion()
    soft.assertion_errors = []
    soft.soft_assert_false(expr)
    assert soft.assertion_errors
    assert len(soft.assertion_errors) == 1


@pytest.mark.unit
@pytest.mark.parametrize('cur, exp', EQUALS_PARAMS_EXTENDED)
def test_soft_assert_equals_valid_data(cur, exp):
    soft = BaseSoftAssertion()
    soft.soft_assert_equals(cur, exp)
    assert not soft.assertion_errors


@pytest.mark.unit
@pytest.mark.parametrize('cur, exp', NOT_EQUALS_PARAMS)
def test_soft_assert_equals_not_valid_data(cur, exp):
    soft = BaseSoftAssertion()
    soft.assertion_errors = []
    soft.soft_assert_equals(cur, exp)
    assert soft.assertion_errors
    assert len(soft.assertion_errors) == 1


@pytest.mark.unit
@pytest.mark.parametrize('cur, exp', NOT_EQUALS_PARAMS)
def test_soft_assert_not_equals_valid_data(cur, exp):
    soft = BaseSoftAssertion()
    soft.soft_assert_not_equals(cur, exp)
    assert not soft.assertion_errors


@pytest.mark.unit
@pytest.mark.parametrize('cur, exp', EQUALS_PARAMS_EXTENDED)
def test_soft_assert_not_equals_not_valid_data(cur, exp):
    soft = BaseSoftAssertion()
    soft.assertion_errors = []
    soft.soft_assert_not_equals(cur, exp)
    assert soft.assertion_errors
    assert len(soft.assertion_errors) == 1


@pytest.mark.unit
@pytest.mark.parametrize('cur, exp', CONTAIN_PARAMS)
def test_soft_assert_in_valid_data(cur, exp):
    soft = BaseSoftAssertion()
    soft.soft_assert_in(cur, exp)
    assert not soft.assertion_errors


@pytest.mark.unit
@pytest.mark.parametrize('cur, exp', NOT_CONTAIN_PARAMS)
def test_soft_assert_in_not_valid_data(cur, exp):
    soft = BaseSoftAssertion()
    soft.assertion_errors = []
    soft.soft_assert_in(cur, exp)
    assert soft.assertion_errors
    assert len(soft.assertion_errors) == 1


@pytest.mark.unit
@pytest.mark.parametrize('cur, exp', NOT_CONTAIN_PARAMS)
def test_soft_assert_not_in_valid_data(cur, exp):
    soft = BaseSoftAssertion()
    soft.soft_assert_not_in(cur, exp)
    assert not soft.assertion_errors


@pytest.mark.unit
@pytest.mark.parametrize('cur, exp', CONTAIN_PARAMS)
def test_soft_assert_not_in_not_valid_data(cur, exp):
    soft = BaseSoftAssertion()
    soft.assertion_errors = []
    soft.soft_assert_not_in(cur, exp)
    assert soft.assertion_errors
    assert len(soft.assertion_errors) == 1


@pytest.mark.unit
@pytest.mark.parametrize('cur, exp', LESS_PARAMS + EQUALS_PARAMS)
def test_soft_assert_less_equal_valid_data(cur, exp):
    soft = BaseSoftAssertion()
    soft.soft_assert_less_equal(cur, exp)
    assert not soft.assertion_errors


@pytest.mark.unit
@pytest.mark.parametrize('cur, exp', NOT_LESS_EQUALS_PARAMS)
def test_soft_assert_less_equal_not_valid_data(cur, exp):
    soft = BaseSoftAssertion()
    soft.assertion_errors = []
    soft.soft_assert_less_equal(cur, exp)
    assert soft.assertion_errors
    assert len(soft.assertion_errors) == 1


@pytest.mark.unit
@pytest.mark.parametrize('cur, exp', LESS_PARAMS)
def test_soft_assert_less_valid_data(cur, exp):
    soft = BaseSoftAssertion()
    soft.soft_assert_less(cur, exp)
    assert not soft.assertion_errors


@pytest.mark.unit
@pytest.mark.parametrize('cur, exp', GREATER_PARAMS)
def test_soft_assert_less_not_valid_data(cur, exp):
    soft = BaseSoftAssertion()
    soft.assertion_errors = []
    soft.soft_assert_less(cur, exp)
    assert soft.assertion_errors
    assert len(soft.assertion_errors) == 1


@pytest.mark.unit
@pytest.mark.parametrize('cur, exp', GREATER_PARAMS + EQUALS_PARAMS)
def test_soft_assert_greater_equal_valid_data(cur, exp):
    soft = BaseSoftAssertion()
    soft.soft_assert_greater_equal(cur, exp)
    assert not soft.assertion_errors


@pytest.mark.unit
@pytest.mark.parametrize('cur, exp', NOT_GREATER_EQUALS_PARAMS)
def test_soft_assert_greater_equal_not_valid_data(cur, exp):
    soft = BaseSoftAssertion()
    soft.assertion_errors = []
    soft.soft_assert_greater_equal(cur, exp)
    assert soft.assertion_errors
    assert len(soft.assertion_errors) == 1


@pytest.mark.unit
@pytest.mark.parametrize('cur, exp', GREATER_PARAMS)
def test_soft_assert_greater_valid_data(cur, exp):
    soft = BaseSoftAssertion()
    soft.soft_assert_greater(cur, exp)
    assert not soft.assertion_errors


@pytest.mark.unit
@pytest.mark.parametrize('cur, exp', LESS_PARAMS)
def test_soft_assert_greater_not_valid_data(cur, exp):
    soft = BaseSoftAssertion()
    soft.assertion_errors = []
    soft.soft_assert_greater(cur, exp)
    assert soft.assertion_errors
    assert len(soft.assertion_errors) == 1


@pytest.mark.unit
def test_soft_assert_regexp_matches_valid_data():
    soft = BaseSoftAssertion()
    soft.soft_assert_regexp_matches('test text #43', r'\w{2}')
    assert not soft.assertion_errors


@pytest.mark.unit
def test_soft_assert_regexp_matches_not_valid_data():
    soft = BaseSoftAssertion()
    soft.assertion_errors = []
    soft.soft_assert_regexp_matches('test text #43', r'\w{3:5}')
    assert soft.assertion_errors
    assert len(soft.assertion_errors) == 1


@pytest.mark.unit
def test_soft_assert_disable_valid_data():
    element = Mock(Element, **{'get_attribute.return_value': True})
    soft = BaseSoftAssertion()
    soft.soft_assert_disable(element)
    assert not soft.assertion_errors


@pytest.mark.unit
def test_soft_assert_disable_not_valid_data():
    element = Mock(Element, **{'get_attribute.return_value': False})
    soft = BaseSoftAssertion()
    soft.assertion_errors = []
    soft.soft_assert_disable(element)
    assert soft.assertion_errors
    assert len(soft.assertion_errors) == 1


@pytest.mark.unit
def test_soft_assert_is_none_valid_data():
    soft = BaseSoftAssertion()
    soft.soft_assert_is_none(None)
    assert not soft.assertion_errors


@pytest.mark.unit
@pytest.mark.parametrize('param', ['none', 0, False, 1, '', [], {}])
def test_soft_assert_is_none_not_valid_data(param):
    soft = BaseSoftAssertion()
    soft.assertion_errors = []
    soft.soft_assert_is_none(param)
    assert soft.assertion_errors
    assert len(soft.assertion_errors) == 1


@pytest.mark.unit
@pytest.mark.parametrize('param', ['none', 0, False, 1, '', [], {}])
def test_soft_assert_is_not_none_valid_data(param):
    soft = BaseSoftAssertion()
    soft.assertion_errors = []
    soft.soft_assert_is_not_none(param)
    assert not soft.assertion_errors


@pytest.mark.unit
def test_soft_assert_is_not_none_not_valid_data():
    soft = BaseSoftAssertion()
    soft.assertion_errors = []
    soft.soft_assert_is_not_none(None)
    assert soft.assertion_errors
    assert len(soft.assertion_errors) == 1


@pytest.mark.unit
@pytest.mark.parametrize('cur, exp', INSTANCE_PARAMS)
def test_soft_assert_is_instance_valid_data(cur, exp):
    soft = BaseSoftAssertion()
    soft.assertion_errors = []
    soft.soft_assert_is_instance(cur, exp)
    assert not soft.assertion_errors


@pytest.mark.unit
@pytest.mark.parametrize('cur, exp', NOT_INSTANCE_PARAMS)
def test_soft_assert_is_instance_not_valid_data(cur, exp):
    soft = BaseSoftAssertion()
    soft.assertion_errors = []
    soft.soft_assert_is_instance(cur, exp)
    assert soft.assertion_errors
    assert len(soft.assertion_errors) == 1


@pytest.mark.unit
@pytest.mark.parametrize('cur, exp', SPACES_PARAMS)
def test_soft_assert_equals_text_with_ignore_spaces_and_register(cur, exp):
    soft = BaseSoftAssertion()
    soft.assertion_errors = []
    soft.soft_assert_equals_text_with_ignore_spaces_and_register(cur, exp)
    assert not soft.assertion_errors


@pytest.mark.unit
def test_soft_assert_schemas_valid_data():
    soft = BaseSoftAssertion()
    soft.soft_assert_schemas(
        current={'key': 1, 'key2': [1, 2, 3], 'key3': '3'},
        expected={'key': int, 'key2': [int], 'key3': str}
    )
    assert not soft.assertion_errors


@pytest.mark.unit
def test_soft_assert_schemas_not_valid_data():
    soft = BaseSoftAssertion()
    soft.soft_assert_schemas(
        current={'key': 1, 'key2': [1, None, 3], 'key3': None},
        expected={'key': str, 'key2': [int], 'key3': dict}
    )
    assert soft.assertion_errors
    assert len(soft.assertion_errors) == 1


@pytest.mark.unit
def test_assert_keys_and_instances_valid_data():
    soft = BaseSoftAssertion()
    soft.assertion_errors = []
    soft.assert_keys_and_instances(
        actual_dict={'key': '1', 'key2': [1, 2, 3], 'key3': 3},
        expected_dict={'key': str, 'key2': list, 'key3': int}
    )
    assert not soft.assertion_errors


@pytest.mark.unit
def test_assert_keys_and_instances_not_valid_data():
    soft = BaseSoftAssertion()
    soft.assertion_errors = []
    soft.assert_keys_and_instances(
        actual_dict={'key': 1, 'key_2': [1, None, 3], 'key_3': None},
        expected_dict={'key': int, 'key_2': list, 'key_3': str}
    )
    assert soft.assertion_errors


@pytest.mark.unit
def test_soft_assert_dicts_with_ignore_types():
    soft = BaseSoftAssertion()
    soft.assertion_errors = []
    soft.soft_assert_dicts_with_ignore_types(
        current={'key': '1'},
        expected={'key': 1, 'key2': 2, 'key3': 3},
    )
    assert not soft.assertion_errors


@pytest.mark.unit
def test_soft_assert_dicts_with_ignore_types_not_valid_data():
    soft = BaseSoftAssertion()
    soft.assertion_errors = []
    soft.soft_assert_dicts_with_ignore_types(
        current={'key': 2},
        expected={'key': 1, 'key_2': 2, 'key_3': 3},
    )
    assert soft.assertion_errors


@pytest.mark.unit
def test_soft_assert_equal_dict_from_clerk_magic():
    soft = BaseSoftAssertion()
    soft.assertion_errors = []
    soft.soft_assert_equal_dict_from_clerk_magic(
        current_dict={'key1': '1', 'key2': 2},
        expected_dict={'key2': 2, 'key1': '1'},
    )
    assert not soft.assertion_errors


@pytest.mark.unit
def test_soft_assert_equal_dict_from_clerk_magic_not_valid_data():
    soft = BaseSoftAssertion()
    soft.assertion_errors = []
    soft.soft_assert_equal_dict_from_clerk_magic(
        current_dict={'key_1': '1', 'key_2': 2},
        expected_dict={'key_2': None, 'key_3': 'AAa'},
    )
    assert soft.assertion_errors


@pytest.mark.unit
def test_request_soft_assert_true():
    soft = RequestSoftAssertion()
    soft.assertion_errors = []
    soft.soft_assert_true(True)
    assert not soft.assertion_errors
